@@ -4,6 +4,7 @@ setMethod("getQuan", "PcaProj", function(obj) obj@n.obs)
 PcaProj <- function (x, ...)
     UseMethod("PcaProj")
 
+#' @export
 PcaProj.formula <- function (formula, data = NULL, subset, na.action, ...)
 {
     cl <- match.call()
@@ -40,8 +41,10 @@ PcaProj.formula <- function (formula, data = NULL, subset, na.action, ...)
     res
 }
 
+#' @export
 PcaProj.default <- function(x, k=0, kmax=ncol(x),
-    scale=FALSE, na.action = na.fail, crit.pca.distances=0.975, trace=FALSE, ...)
+    scale=FALSE, na.action = na.fail, crit.pca.distances=0.975,
+    trace=FALSE, ...)
 {
 
     cl <- match.call()
@@ -61,7 +64,8 @@ PcaProj.default <- function(x, k=0, kmax=ncol(x),
     if((k <- floor(k)) < 0)
         k <- 0
     else if(k > kmax) {
-        warning(paste("The number of principal components k = ", k, " is larger then kmax = ", kmax, "; k is set to ", kmax,".", sep=""))
+        warning(paste("The number of principal components k = ", k,
+        " is larger then kmax = ", kmax, "; k is set to ", kmax,".", sep=""))
         k <- kmax
     }
     if(k != 0)
@@ -71,10 +75,8 @@ PcaProj.default <- function(x, k=0, kmax=ncol(x),
         if(trace)
             cat("The number of principal components is defined by the algorithm. It is set to ", k,".\n", sep="")
     }
-######################################################################
 
-    if(is.logical(scale))
-    {
+    if(is.logical(scale)) {
         scale <- if(scale) sd else  NULL
     }
     out <- PCAproj(data, k, scale=scale, ...)
@@ -88,7 +90,6 @@ PcaProj.default <- function(x, k=0, kmax=ncol(x),
 ##    scores <- predict(out)
 ##    scores   <- as.matrix(scores[, 1:k])
     scores <- (data - matrix(rep(center, nrow(data)), nrow = nrow(data), byrow = TRUE)) %*% loadings
-
     eigenvalues  <- (sdev^2)[1:k]
 
 ######################################################################
